// SNF4CGP/Command.hpp
// Copyright (C) 2009 ARM Research Labs, LLC
// See www.armresearch.com for more information.
//
// The Command class describes what CGP is asking us to do.

#ifndef IncludedCommand
#define IncludedCommand

#include <string>

using namespace std;

class Command {                                                                 // Parsed Command Objects
  public:

    enum CommandType {                                                          // Commands have these types:
        UNKNOWN,                                                                // We don't know this command.
        QUIT,                                                                   // It is time to stop the app.
        WAKE,                                                                   // Report the app is now awake.
        INTF,                                                                   // Report the interface version.
        FILE                                                                    // Scan this file.
    };

    unsigned int Number;                                                        // Commands have a serial number
    CommandType Type;                                                           // Commands have a type.
    string Data;                                                                // Other command data goes here.

    Command::Command() :                                                        // Ordinary constructor.
      Number(0),
      Type(UNKNOWN),
      Data("") {
    }

    Command::Command(const Command& R) :                                        // Copy constructor.
      Number(R.Number),
      Type(R.Type) {
        Data = R.Data;
    }

    Command& operator=(const Command& R) {                                      // We need an assignment operator.
        Number = R.Number;
        Type = R.Type;
        Data = R.Data;
        return (*this);
    }
};

#endif
