// SNF4CGP/ExecutiveProcess.cpp
// Copyright (C) 2009 ARM Research Labs, LLC.
// See www.armresearch.com for more information.

#include "ExecutiveProcess.hpp"

#include "../CodeDweller/faults.hpp"

#include <iostream>

using namespace std;

void ExecutiveProcess::initializeOutput() {
    Output.init(Jobs);
}

void ExecutiveProcess::initializeJobPool() {
    Jobs.init(ConfigInfo, Output);
}

void ExecutiveProcess::initializeWorkerPool() {
    Workers.init();
}

void ExecutiveProcess::shutdownWorkerPool() {
    Workers.stop();
}

void ExecutiveProcess::shutdownJobPool() {
    Jobs.stop();
}

void ExecutiveProcess::shutdownOutput() {
    Output.stop();
}

void ExecutiveProcess::dispatchCommand(Command& C) {                            // Job + Worker + Command; go!
    Job& J = Jobs.grab();
    J.setCommand(C);
    Worker& W = Workers.grab();
    W.doJob(J);
}

ExecutiveProcess::ExecutiveProcess(const string& Version, const string& Config) :   // Simple construction.
  VersionInfo(Version),
  ConfigInfo(Config),
  QuitJobNumber(0) {}

ExecutiveProcess::~ExecutiveProcess() {                                         // Deal with uncerimoneous shutdowns.
}

void ExecutiveProcess::doStartup() {                                            // Startup happens in this sequence.
    initializeJobPool();
    initializeOutput();
    initializeWorkerPool();
}

void ExecutiveProcess::doProcessing() {                                         // Command processing happens here.
    Command C;
    C.Type = Command::WAKE;                                                     // Create the WAKE command to announce
    C.Data = VersionInfo;                                                       // that we are alive (and who we are)
    dispatchCommand(C);                                                         // and dispatch a job with it.
                                                                                // Then process all incoming commands:
    for(;0 == QuitJobNumber;) {                                                 // Loop until we have a QUIT job.
        C = Input.getCommand();                                                 // Get a command from the input.
        if(Command::QUIT == C.Type) QuitJobNumber = C.Number;                   // QUIT command means it's time to stop.
        else dispatchCommand(C);                                                // Dispatch a job for all other commands.
    }
}

void ExecutiveProcess::doShutdown() {                                           // Shutdown happens in this sequence.
    shutdownWorkerPool();
    shutdownOutput();
    shutdownJobPool();
    cout << QuitJobNumber << " OK" << endl;
}
